// KESIReader
// Copyright 2005 Kurzweil Educational Systems, Inc. 
// Last modified 4/26/05 md

var gBrowser = null;
var gInStream = null;
var gOutStream = null;
var gHighlightDocument = null;
var gHighlightAltNode = null;
var gHighlightAltNodeBorder = null;
var gReadAltText = false;
var gIsMac = false;

// Default values for Preferences
var gPort = 1007;	// 5002 if Macintosh
var gSuppressDebuggingAlerts = true;

var socketListener = {
	onStartRequest: function(request, context) {
		//alert("onStartRequest");
	},
	onStopRequest: function(request, context, status) {
		//alert("onStopRequest, context: " + context + " status: " + status);
		highlightReset();
		clearAllHighlights();
		closeConnection();
		if (gBrowser) gBrowser.removeProgressListener(browserProgressListener);
	},
	onDataAvailable: function(request, context, inputStream, offset, count) {
		var dat = gInStream.read(count);
		doCommand(dat);
	}
};

function doCommand(cmd)
{
	for (var n = 0; n < cmd.length; ++n) {
		var ch = cmd.charAt(n);
		if (ch == "h") {
			// highlight
			var selStart = 0;
			var selLength = 0;
			var selGroup = cmd.charAt(++n);
			var color = "";
			var nArg = 0;
			var val = 0;
			for (; ++n < cmd.length;) {
				ch = cmd.charAt(n);
				if (ch == ";") {
					break;
				} else if (ch == ",") {
					if (nArg == 0) selStart = val;
					if (nArg == 1) selLength = val;
					++nArg;
					val = 0;
				} else {
					if (nArg < 2) {
						val = val * 10 + (cmd.charCodeAt(n) - 48);
					} else {
						color = color + ch;
					}
				}
			}
			//window.focus();
			highlightKESIText(content.document, selStart, selLength, color, selGroup);
		} else if (ch == "u") {
			clearHighlightGroup(gHighlightDocument, cmd.charAt(++n));
		} else if (ch == "r") {
			sendSelection("r");
		} else if (ch == "G") {
			var str = gatherText(content.document);
			highlightReset();
			gHighlightDocument = content.document;	// 6037
			clearAllHighlights();			// 6037
			str = toUTF8(str);
			sendSocketMessage("t" + str.length + "," + str);
		} else if (ch == "A") {
			gReadAltText = true;
		} else if (ch == "a") {
			gReadAltText = false;
		} else if (ch == "I") {
			kesiInit2();
		} else if (ch == "d") {
			var url = "";
			for (; ++n < cmd.length;) {
				ch = cmd.charAt(n);
				if (ch == ";") break;
				url = url + ch;
			}
			var win = window.open(url, "KESIDictionary",              "left=100,top=100,height=400,width=600,location=no,toolbar=no,status=no,menubar=no,scrollbars=yes,resizable=yes");
			if (win) win.focus();
		}
	}
}


// This succeeds even if K3000 is not there. (but sendSocketMessage will fail 
// eventually, but not necessarily the first time)

function initSocket()
{
	if (gInStream) return;
	
	var host = "127.0.0.1";
	
	try {
		var transportService =
		Components.classes["@mozilla.org/network/socket-transport-service;1"]
			.getService(Components.interfaces.nsISocketTransportService);
		var transport = transportService.createTransport(null, 0, host, gPort, null);
		gOutStream = transport.openOutputStream(0, 0, 0);
	} catch (ex) {
		kesiAlert("initSocket OutStream: " + ex);
	}
	
	try {
		var stream = transport.openInputStream(0, 0, 0);
		gInStream = Components.classes["@mozilla.org/scriptableinputstream;1"]
			.createInstance(Components.interfaces.nsIScriptableInputStream);
		gInStream.init(stream);
		var pump = Components.
		classes["@mozilla.org/network/input-stream-pump;1"]
			.createInstance(Components.interfaces.nsIInputStreamPump);
		pump.init(stream, -1, -1, 0, 0, false);
		pump.asyncRead(socketListener, null);
	} catch (ex) {
		kesiAlert("initSocket InStream: " + ex);
	}
}


// if this is called twice rapidly (e.g. click and LocationChange) then
// just one concatenated string may arrive at the server.

function sendSocketMessage(data)
{
	try {
		if (!gOutStream) return false;
		gOutStream.write(data, data.length);
	} catch (ex) {
		//alert("sendSocketMessage: " + ex + " data: " + data);
		return false;
	}
	return true;
}


// these messages do not happen when user's starting pages are loading

var browserProgressListener = {
	onStateChange : function(aWebProgress, aRequest, aStateFlags, aStatus)
	{
		// Components.interfaces.nsIWebProgressListener.
		// STATE_STOP = 0x0010   STATE_START = 0x0001
		//sendSocketMessage("dStateChange " + aStateFlags + ";");
		if (aStateFlags & 1) {
			highlightReset();
			clearAllHighlights();
			sendSocketMessage("L");
		} else if (aStateFlags & 0x10) {
			sendSocketMessage("p");
		}
	},
	
	onLocationChange : function(aWebProgress, aRequest, aLocation) {
		highlightReset();
		clearAllHighlights();
		sendSocketMessage("b");
	},
	
	QueryInterface : function(aIID) {
		if (aIID.equals(Components.interfaces.nsIWebProgressListener) ||
		    aIID.equals(Components.interfaces.nsISupportsWeakReference) ||
		    aIID.equals(Components.interfaces.nsISupports))
			return this;
		throw Components.results.NS_NOINTERFACE;
	}
};


// called for mouse clicks
// (event.button = 1 is left, 2 is right)

function browserMouseUpHandler(event)
{
	try {
		if (gOutStream) {
			if (event && event.target && event.target.localName == "scrollbar") {
				sendSocketMessage("s");
			} else {
				sendSelection("c");
			}
		}
	} catch (ex) {
		kesiAlert ("browserMouseUpHandler: " + ex);
	}
}

function browserMouseDownHandler(event)
{
	if (gOutStream) sendSocketMessage("C");
}

function sendSelection(cmdChar)
{
	try {
		var selStartLen = selectionOffset(content.document, focusedSelection());
		sendSocketMessage(cmdChar + selStartLen[0] + "," + selStartLen[1] + ";");
	} catch (ex) {
		kesiAlert ("sendSelection: " + ex);
	}
}


// for debugging

/*
function evalPrompt()
{
	for (var i = 1; i < 10000; ++i) {
		var str = prompt();
		if (!str || str == "") break;
		try {
			alert(str + "\n" + eval(str));
		} catch (ex) {
			alert("error\n" + str + "\n" + ex);
		}
	}
}
*/


// for debugging

function printable(pstr)
{
	try {
		var res = "";
		for (var n = 0; n < pstr.length; ++n) {
			//if (pstr.charCodeAt(n) < 32 || pstr.charCodeAt(n) > 127) res = res + "&" + pstr.charCodeAt(n);
			if (pstr.charCodeAt(n) < 32) res = res + "&" + pstr.charCodeAt(n);
			res = res + pstr.charAt(n);
		}
		return res;
	} catch (ex) {
		return "<not a string>";
	}
}

function kesiAlert(msg)
{
	if (!gSuppressDebuggingAlerts) {
		alert("KESIReader error at " + msg);
		dump("KESIReader error at " + msg);
	}
}

function toUTF8(instr)
{
	try {
		var outstr = "";
		var cc;
		var c;
		for (var n = 0; n < instr.length; ++n) {
			cc = instr.charCodeAt(n);
			if (cc <= 127) {
				c = instr.charAt(n);
			} else if (cc == 160) {	// 6321
				c = ' ';
			} else if (cc <= 2047) {
				c = String.fromCharCode(192 + (cc>>6), 128 + (cc&63));
			} else {
				c = String.fromCharCode(224 + (cc>>12), 128 + ((cc>>6)&63), 128 + (cc&63));
			}
			outstr += c;
		}
	} catch (ex) {
		kesiAlert ("toUTF8: " + ex);
	}
	return outstr;
}



// "document" is an HTMLDocument object

function gatherText(document)
{
	var text = "";
	if (!document) return text;
	var nName;
	var altText
	
	try {
		var node = document.firstChild;
		while(node) {
			nName = node.nodeName.toUpperCase();
			// See if this node is text.
			if (nName == "#TEXT") {
				// replace newlines with spaces
				text += node.data.replace( /\n|\r/g, " " );
				//            } else if (nName == "LI") {
			} else if (node.nodeType == 1 && nName == "IMG") {
				if (gReadAltText) {
					altText = node.getAttribute("alt");
					if (altText && altText != "") {
						text += "\r\n" + altText.replace( /\n|\r/g, " " ) + "\r\n";
					}
				}
			} else if (node.nodeType == 1 && nName == "INPUT") {
				if (gReadAltText) {
					altText = getInputNodeStr(node);
					if (altText != "" ) {
						text += "\r\n" + altText + "\r\n";
					}
				}
			} else if (!isInline(nName)) {
				text += "\r\n";
			}
			
			if (node.hasChildNodes() &&
			    (!node.style || node.style.visibility != 'hidden') &&
			    nName != "HEAD" && 	// nothing interesting in HEAD
			    nName != "SCRIPT" && nName != "NOSCRIPT" &&
			    nName != "INPUT" && nName != "STYLE" && nName != "NOFRAMES" &&
			    nName != "FRAME" && nName != "IFRAME" && nName != "SELECT")
			{
				node = node.firstChild;
			} else {
				if (nName == "FRAME" || nName == "IFRAME") {
					//Frames do not have children (except script) and the child document 
					//  does not have a parent: need to recurse
					text += "\r\n" + gatherText(node.contentDocument);
				}
				
				// No children, try next sibling.
				if (node.nextSibling) {
					node = node.nextSibling;
				} else {
					// Loop upward until a sibling is found!
					var addCRLF = false;
					while ((node = node.parentNode)) {
						// Catch case like:  para text <H1>Header</H1> second para
						if (!isInline(node.nodeName.toUpperCase())) addCRLF = true;
						if (node.nextSibling) {
							node = node.nextSibling;
							break;
						}
					}
					if (addCRLF) text += "\r\n";
				}
			}
		}	//while
	} catch (ex) {
		kesiAlert("gatherText: " + ex + " Node: " + node + " Name: " + nName);
	}
	return text;
}

function getInputNodeStr(node)
{
	var altText = "";
	try {
		if (node.getAttribute("type") == "submit") altText = node.getAttribute("value");
		if (!altText || altText == "") altText = node.getAttribute("alt");
		if (!altText) altText = "";
	} catch (ex) {
		kesiAlert("getInputNodeStr: " + ex);
	}
	return altText.replace( /\n|\r/g, " " );  // replace newlines with spaces
}


function isInline(nName)
{
	switch (nName) {
		case "A":
		case "ADDRESS":
		case "BIG":
		case "BLINK":
		case "B":
		case "CITE":
		case "CODE":
		case "DFN":
		case "EM":
		case "FONT":
		case "I":
		case "KBD":
		case "KEYGEN":
		case "NOBR":
		case "S":
		case "SAMP":
		case "SMALL":
		case "SPACER":
		case "SPAN":
		case "STRIKE":
		case "STRONG":
		case "SUB":
		case "SUP":
		case "TT":
		case "U":
		case "VAR":
		case "WBR":
		case "#TEXT":
		case "#COMMENT":
		case "FRAME":	// treat these like inline
		case "FRAMESET":
		case "IFRAME":
		case "#DOCUMENT":
			return true;
		default:
			return false;
	}
}




// Remember a starting point to speed up successive highlighting calls
var lastHLPos = -1;
var lastHLNode = null;
var lastHLDocument;
var lastHLParentDocument;
var lastHLDepth;
var frameStack = new Array(1);
var frameDepth = 0;


// Reset the memory of the optimization starting point

function highlightReset()
{
	lastHLPos = -1;
	lastHLNode = null;
	lastHLDocument = null;
	lastHLParentDocument = null;
	frameDepth = 0;
	frameStack.length = 0;
}


// selStart is 0-based
// "document" is an HTMLDocument object
// If the selection (or groups) spans several frames, only the last piece will survive
// If selLength = 0, position the caret (and scroll the parent if group B)

function highlightKESIText(parentDocument, selStart, selLength, selColor, selGroup)
{
	var pos;
	var len;
	var node;
	var nextSib;
	var nName;
	var depth;
	var baseID = "KESIHighlight" + selGroup;
	var id = 0;
	var spanNode = null;
	var skipChildren = false;
	var groupA = (selGroup == "A");	// group A is single word
	var selLengthRem = selLength;
	var document;
	var altText;
	var altLen;
	
	try {
		//if (true) {	// Disable speedup code for DEBUGGING
		if (lastHLPos < 0 || selStart < lastHLPos || parentDocument != lastHLParentDocument) {
			// start at beginning
			document = parentDocument;
			pos = 0;
			node = document.firstChild;
			depth = 1;
			frameDepth = 0;
			frameStack.length = 0;   
		} else {
			// start where we left off
			document = lastHLDocument;
			pos = lastHLPos;
			node = lastHLNode;
			depth = lastHLDepth;
		}
		lastHLParentDocument = parentDocument;
		
		while (node && depth > 0) {
			nName = node.nodeName.toUpperCase();
			nextSib = node.nextSibling;
			if (nName == "#TEXT") {
				len = node.data.length;
				if (pos + len > selStart) {
					var colorNode;
					
					//highlightAltNode(null);
					if (document != gHighlightDocument) {
						clearAllHighlights();
						gHighlightDocument = document;
						id = 0;
					}
					
					if (selLength == 0) break;
					
					if (pos >= selStart) {
						if (pos + len <= selStart + selLengthRem) {
							// selection includes entire node
							colorNode = node;
							selLengthRem = selStart + selLengthRem - (pos + len);
							selStart = pos + len;
						} else {
							// selection is front of node
							var nl = selStart + selLengthRem - pos;
							if (nl <= 0) break;		// could be alt text before the node
							node.splitText(nl);
							colorNode = node;
							nextSib = null;
							depth = -10000;	// force a break
						}
					} else {
						if (pos + len > selStart + selLengthRem) {
							// selection is middle chunk of node
							colorNode = node.splitText(selStart - pos);
							node = colorNode.splitText(selLengthRem);
							nextSib = null;
							depth = -10000;	// force a break
						} else {
							// selection is tail of node
							colorNode = node.splitText(selStart - pos);
							node = colorNode;
							selLengthRem -= len - (selStart - pos);
							selStart = pos + len;
						}
					}
					
					var colorClone = colorNode.cloneNode(true);
					spanNode = document.createElement("span");
					++id;
					spanNode.setAttribute("id", baseID + id);
					spanNode.style.backgroundColor = selColor;
					spanNode.appendChild(colorClone);
					colorNode.parentNode.replaceChild(spanNode, colorNode);
					
					if (selLengthRem <= 0) depth = -10000;
					node = spanNode;
					skipChildren = true;	// don't look at stuff we just added!
				}
				
				pos += len;
				if (depth < 0) break;
			} else if (node.nodeType == 1 && nName == "IMG") {
				if (gReadAltText) {
					altText = node.getAttribute("alt");
					if (altText) {
						altLen = altText.length;
						if (altLen > 0) {
							//if (!groupA && pos + 2 >= selStart && pos + altLen + 2 <= selStart + selLengthRem) {
							//    highlightAltNode(node);
							//}
							pos +=  altLen + 4;
						}
					}
				}
			} else if (node.nodeType == 1 && nName == "INPUT") {
				if (gReadAltText) {
					altText = getInputNodeStr(node);
					altLen = altText.length;
					if (altLen > 0) {
						//if (!groupA && pos + 2 >= selStart && pos + altLen + 2 <= selStart + selLengthRem) {
						//    highlightAltNode(node);
						//}
						pos +=  altLen + 4;
					}
				}
			} else if (!isInline(nName)) {
				// Save a possible place to start again next time
				//   (Note that this incidentally avoids the temporary spans we are adding)
				lastHLPos = pos;
				lastHLNode = node;
				lastHLDocument = document;
				lastHLDepth = depth;
				
				pos += 2;
			}
			
			if (node.hasChildNodes() && !skipChildren &&
			    (!node.style || node.style.visibility != 'hidden') &&
			    nName != "HEAD"     &&  nName != "SCRIPT" &&
			    nName != "NOSCRIPT" &&  nName != "INPUT"  &&
			    nName != "STYLE" && nName != "FRAME" && nName != "NOFRAMES" &&
			    nName != "IFRAME" && nName != "SELECT") {
				depth++;
				node = node.firstChild;
			} else {
				skipChildren = false;
				if (nName == "FRAME" || nName == "IFRAME") {
					pos += 2;
					depth++;
					frameStack[frameDepth++] = node;
					document = node.contentDocument;
					node = document.firstChild;
				} else if (nextSib) {
					node = nextSib;
				} else {
					var addCRLF = false;
					while (depth-- > 0) {
						node = node.parentNode;
						if (!node) {
							if (frameDepth <= 0) break;		// 6298 #document has null parentNode
							node = frameStack[--frameDepth];
							frameStack[frameDepth] = null;
							if (!node) break;
							if (addCRLF) {	//6349
								addCRLF = false;
								pos += 2;
							}
							document = node.ownerDocument;
							lastHLPos = -1;			// 6298
						}
						if (!isInline(node.nodeName.toUpperCase())) addCRLF = true;
						if (node.nextSibling) {
							node = node.nextSibling;
							break;
						}
					}
					if (addCRLF) pos += 2;
				}
			}
		}  //end of while loop
		
		if (selLength == 0 && !spanNode && node) {	//set caret
			var selection = focusedSelection();
			selection.removeAllRanges();
			var range = parentDocument.createRange();
			var offset = selStart - pos;
			if (offset < 0) offset = 0;
			try {	// this can fail for degenerate documents
				range.setStart(node, offset);
				range.setEnd(node, offset);
				selection.addRange(range);
				setSelectionVisibility(true);
				spanNode = node;
			} catch (ex) {}
		}
		if (spanNode) {
			if (!groupA && spanNode.parentNode && spanNode.parentNode.scrollIntoView)
				spanNode.parentNode.scrollIntoView(false);
			if (spanNode.scrollIntoView) spanNode.scrollIntoView(false);
		}
	} catch (ex) {
		kesiAlert("highlightKESIText: " + ex);
		//kesiAlert("selStart: " + selStart + " selLength: " + selLength);
		//kesiAlert("pos: " + pos + " len: " + len + " data: " + printable(node.data));
	}
	return pos;
}

/*
function highlightAltNode(node)
{
	try {
		if (gHighlightAltNode) {
			gHighlightAltNode.setAttribute("border", gHighlightAltNodeBorder);
		}
		gHighlightAltNode = node;
		if (node) {
			gHighlightAltNodeBorder = node.getAttribute("border");
			node.setAttribute("border", Number(gHighlightAltNodeBorder) + 2);
		}
	} catch (ex) {
		kesiAlert("highlightAltNode: " + ex);
	}
}
*/

//see http://lxr.mozilla.org/seamonkey/source/xpfe/browser/resources/content/viewPartialSource.js#300
//    http://lxr.mozilla.org/seamonkey/source/content/base/public/nsISelectionController.idl

//also http://lxr.mozilla.org/seamonkey/source/content/base/src/nsSelection.cpp#7396

function setSelectionVisibility(caretVisible)
{
	try {
		var selCon = gBrowser.docShell.QueryInterface(Components.interfaces.nsIInterfaceRequestor)
			.getInterface(Components.interfaces.nsISelectionDisplay)
			.QueryInterface(Components.interfaces.nsISelectionController);
		
		if (caretVisible) {
			selCon.setCaretEnabled(caretVisible);
			selCon.setDisplaySelection(Components.interfaces.nsISelectionController.SELECTION_ON);
			//selCon.setCaretVisibilityDuringSelection(false);
		} else {
			selCon.setCaretEnabled(caretVisible);
			selCon.setDisplaySelection(Components.interfaces.nsISelectionController.SELECTION_HIDDEN);
			//selCon.setCaretVisibilityDuringSelection(false);
		}
	} catch(ex) {
		kesiAlert("setSelectionVisibility: " + ex);
	}
}


// Get selection object of the frame that is in focus.
// See http://developer-test.mozilla.org/docs/Safely_accessing_content_DOM_from_chrome

function focusedSelection()
{
	try {
		var focusedWindow = new XPCNativeWrapper(document.commandDispatcher.focusedWindow, 'document', 'getSelection()');
		if (focusedWindow) {
			var selection = focusedWindow.getSelection();
			if (selection) return selection;
		}
	} catch (ex) {
		kesiAlert("focusedSelection: " + ex);
	}
	return null;
}


// return [offset, length] of a Mozilla selection
// offset is zero based. length is -1 if there is no selection.
// document is an HTMLDocument object

function selectionOffset(document, selection)
{
	var node;
	var pos = 0;
	var len;
	var nextSib;
	var nName;
	var depth = 1;
	var selStart = -1;
	var selEnd = -1;
	var selRange;
	var addCRLF = false;
	var altText;
	
	try {
		if(!selection || selection.rangeCount <= 0 || !(selRange = selection.getRangeAt(0))) 
			return [-1, -1];
		
		node = document.firstChild;
		
		while (node && depth > 0) {
			if (node == selRange.startContainer) selStart = pos + selRange.startOffset;
			if (node == selRange.endContainer) selEnd = pos + selRange.endOffset;
			if (selStart >= 0 && selEnd >= 0) break;
			
			nName = node.nodeName.toUpperCase();
			nextSib = node.nextSibling;
			if (nName == "#TEXT") {
				len = node.data.length;
				pos += len;
			} else if (node.nodeType == 1 && nName == "IMG") {
				altText = node.getAttribute("alt");
				if (altText && altText != "" && gReadAltText) {
					pos += altText.length + 4;
				}
			} else if (node.nodeType == 1 && nName == "INPUT") {
				altText = getInputNodeStr(node);
				if (altText != "" && gReadAltText) {
					pos += altText.length + 4;
				}
			} else if (!isInline(nName)) {
				pos += 2;
			}
			
			if (node.hasChildNodes() &&
			    (!node.style || node.style.visibility != 'hidden') &&
			    nName != "HEAD"     &&  nName != "SCRIPT" &&
			    nName != "NOSCRIPT" &&  nName != "INPUT"  &&
			    nName != "STYLE" && nName != "FRAME" && nName != "NOFRAMES" &&
			    nName != "IFRAME" && nName != "SELECT") {
				depth++;
				node = node.firstChild;
			} else {
				if (nName == "FRAME" || nName == "IFRAME") {
					pos += 2;
					var framePos = selectionOffset(node.contentDocument, selection);
					if (framePos[0] >= 0) return [pos + framePos[0], framePos[1]];
					pos -= framePos[0] + 1;
				}
				
				if (nextSib) {
					node = nextSib;
				} else {
					addCRLF = false;
					while ((node = node.parentNode) && depth-- > 0) {
						if (!isInline(node.nodeName.toUpperCase())) addCRLF = true;
						if (node.nextSibling) {
							node = node.nextSibling;
							break;
						}
					}
					if (addCRLF) pos += 2;
				}
			}
		}
	} catch (ex) {
		kesiAlert("selectionOffset: " + ex);
	}
	
	if (selStart >= 0) {
		if (selEnd < 0) selEnd = selStart;	// should not happen
		if (selEnd >= selStart) {
			return [selStart, selEnd - selStart];
		} else {
			return [selEnd, selStart - selEnd];
		}
	}
	
	return [-pos - 1, 0];	// selection not found (in this frame) (-1 adjusts for zero case)
}


function clearAllHighlights()
{
	//highlightAltNode(null);
	if (gHighlightDocument) {
		while (clearHighlightGroup(gHighlightDocument, "A") > 1);
		while (clearHighlightGroup(gHighlightDocument, "B") > 1);
		gHighlightDocument = null;
	}
}

// Note that highlight ID numbers must always be used in continuous runs,
// since this stops once an ID number is not found. Returns number of first ID not found.

function clearHighlightGroup(document, selGroup)
{
	var id = 1;
	try {
		//if (selGroup != "A") highlightAltNode(null);
		if (!document) return;
		var baseID = "KESIHighlight" + selGroup;
		for (; true; ++id) {
			var oldSpan = document.getElementById(baseID + id);
			if (!oldSpan) break;
			var parent = oldSpan.parentNode;
			parent.replaceChild(oldSpan.childNodes[0], oldSpan);
			parent.normalize();
		}
	} catch (ex) {
		kesiAlert("clearHighlight: " + ex);
	}
	return id;
}


var k2init = false;

function kesiInit2()
{
	if (k2init) return;
	k2init = true;
	
	try {
		window.addEventListener("focus", winFocus, false);
		highlightReset();
	} catch (ex) {
		kesiAlert("kesiInit2: " + ex);
	}
}

function kesiInit3()
{
	try {
		gBrowser = top.document.getElementById("content");
		gBrowser.addEventListener("mouseup", browserMouseUpHandler, false);
		if (gIsMac) gBrowser.addEventListener("mousedown", browserMouseDownHandler, false);

		gBrowser.addProgressListener(browserProgressListener, 
			Components.interfaces.nsIWebProgress.NOTIFY_STATE_ALL | 
			Components.interfaces.nsIWebProgress.NOTIFY_LOCATION);
	} catch (ex) {
		kesiAlert("kesiInit3: " + ex);
	}
}

function winFocus()
{
	if (gOutStream) sendSocketMessage("f");
}


// gets called once during startup, then again if "New window"

function windowLoadHandler()
{
	initSocket();
	sendSocketMessage("w");
	if (sendSocketMessage("a")) kesiInit3();
}

function winUnload()
{
	try {
		sendSocketMessage("u");
		highlightReset();
		clearAllHighlights();
		closeConnection();
		if (gBrowser) gBrowser.removeProgressListener(browserProgressListener);
		gBrowser = null;
		k2init = false;
	} catch (ex) {
	}
}

function closeConnection()
{
	if (gInStream) {
		gInStream.close();
		gInStream = null;
	}
	if (gOutStream) {
		gOutStream.close();
		gOutStream = null;
	}
}

function getPrefs()
{
	try {
		var prefObj = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService);
		var Branch = prefObj.getBranch("kesireader.");
		if (Branch.prefHasUserValue("port")) {
			gPort = Branch.getIntPref("port");
		} else {
			if (gIsMac) gPort = 5002;
			Branch.setIntPref("port", gPort);
		}
		if (Branch.prefHasUserValue("suppressDebuggingAlerts")) {
			gSuppressDebuggingAlerts = Branch.getBoolPref("suppressDebuggingAlerts");
		} else {
			Branch.setBoolPref("suppressDebuggingAlerts", gSuppressDebuggingAlerts);
		}
	} catch(ex) {
		kesiAlert("getPrefs: " + ex);
	}
}

function kesiInit1()
{
	try {
		var agent = navigator.userAgent.toLowerCase();
		gIsMac = (agent.indexOf("mac") != -1);
		getPrefs();
		window.addEventListener("load", windowLoadHandler, false);
		window.addEventListener("unload", winUnload, false);
	} catch(e) {
		kesiAlert("kesiInit1: " + ex);        
	}
}

kesiInit1();	// trigger initializations
